/* binding_ctrl.cpp - helper functions for binding wxWindow to
    wxPg::Row_binding

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/binding/binding_ctrl.h>
#include<wx/calctrl.h>
#include<wx/combobox.h>
#include<wx/combo.h>
#include<wx/richtext/richtextctrl.h>
#include<wx/textctrl.h>
#include<wx/spinctrl.h>
#include<wx/stattext.h>
#include<wx/hyperlink.h>
#include<wx/datectrl.h>
#include<wx/checkbox.h>
#include<wx/gauge.h>
#include<wx/spinbutt.h>
#include<wx/radiobox.h>
#include<wx/vlbox.h>
#include<wx/html/htmlwin.h>


void
wxPg::update_calendar_ctrl(wxWindow *calendar, const wxString &field,
                           wxPg::Data_row_provider *row){
    static_cast<wxCalendarCtrl*>(calendar)->SetDate(row->get_date(field));
}


void
wxPg::update_combo_box(wxWindow *combo, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxComboBox*>(combo)->SetValue(row->get_str(field));
}


void
wxPg::update_combo_ctrl(wxWindow *combo, const wxString &field,
                        wxPg::Data_row_provider *row){
    static_cast<wxComboCtrl*>(combo)->SetValue(row->get_str(field));
}


void
wxPg::update_rich_text_ctrl(wxWindow *text_ctrl, const wxString &field,
                            wxPg::Data_row_provider *row){
    static_cast<wxRichTextCtrl*>(text_ctrl)->SetValue(row->get_str(field));
}


void
wxPg::update_text_ctrl(wxWindow *text_ctrl, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxTextCtrl*>(text_ctrl)->SetValue(row->get_str(field));
}


void
wxPg::update_spin_ctrl(wxWindow *spin, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxSpinCtrl*>(spin)->SetValue(row->get_int(field));
}


void
wxPg::update_static_text(wxWindow *static_text, const wxString &field,
                         wxPg::Data_row_provider *row){
    static_cast<wxStaticText*>(static_text)->SetLabel(row->get_str(field));
}


void
wxPg::update_hyperlink_ctrl(wxWindow *hyperlink, const wxString &field,
                            wxPg::Data_row_provider *row){
    static_cast<wxHyperlinkCtrl*>(hyperlink)->SetURL(row->get_str(field));
}


void
wxPg::update_date_picker_ctrl(wxWindow *date_ctrl, const wxString &field,
                              wxPg::Data_row_provider *row){
    static_cast<wxDatePickerCtrl*>(date_ctrl)->SetValue(row->get_date(field));
}


void
wxPg::update_check_box(wxWindow *check_box, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxCheckBox*>(check_box)->SetValue(row->get_bool(field));
}


void
wxPg::update_gauge(wxWindow *gauge, const wxString &field,
                   wxPg::Data_row_provider *row){
    static_cast<wxGauge*>(gauge)->SetValue(row->get_int(field));
}


void
wxPg::update_spin_button(wxWindow *spin, const wxString &field,
                         wxPg::Data_row_provider *row){
    static_cast<wxSpinButton*>(spin)->SetValue(row->get_int(field));
}


void
wxPg::update_item_container(wxWindow *container, const wxString &field,
                            wxPg::Data_row_provider *row){
    //need dynamic_cast because of multiple inheritance
    wxItemContainer *cont = dynamic_cast<wxItemContainer*>(container);
    if(cont)
        cont->SetStringSelection(row->get_str(field));
}


void
wxPg::update_radio_box(wxWindow *box, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxRadioBox*>(box)->SetSelection(row->get_int(field));
}


void
wxPg::update_vlist_box(wxWindow *list, const wxString &field,
                       wxPg::Data_row_provider *row){
    static_cast<wxVListBox*>(list)->SetSelection(row->get_int(field));
}


void
wxPg::update_html_window(wxWindow *html, const wxString &field,
                         wxPg::Data_row_provider *row){
    static_cast<wxHtmlWindow*>(html)->SetPage(row->get_str(field));
}
