/* grid_binding.cpp - binding wxGrid to display the values of some
    fields in wxPg::Result_data

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/binding/grid_binding.h>

wxPg::Grid_binding::Grid_binding()
    :grid_(NULL), data_(NULL)
{}


wxPg::Grid_binding::Grid_binding(wxGrid *grid)
    :grid_(grid), data_(NULL)
{}


wxPg::Grid_binding::Grid_binding(wxPg::Result_data_provider *data)
    :grid_(NULL), data_(data)
{}


wxPg::Grid_binding::Grid_binding(wxGrid *grid,
                                 wxPg::Result_data_provider *data)
    :grid_(grid), data_(data)
{}


void
wxPg::Grid_binding::bind(int col_idx, const wxString &field){
    this->col_bindings_[col_idx] = field;
}


void
wxPg::Grid_binding::bind(const wxString &field){
    if(this->col_bindings_.size()){
        int new_col_idx = (--(this->col_bindings_.end()))->first + 1;
        this->col_bindings_[new_col_idx] = field;
    }else this->col_bindings_[0] = field;
}


void
wxPg::Grid_binding::unbind(wxWindow *c){}


void
wxPg::Grid_binding::update_UI(){
    if(!this->grid_ || !this->data_)
        return;
    this->grid_->BeginBatch();
    this->prepare_grid_();
    int current_rec = this->data_->current();
    int total_recs = this->data_->rows();
    int cursor = 0;
    this->data_->current(0);
    do{
        if(this->col_bindings_.size())
            this->fill_row_(cursor);
        else this->fill_row_all_(cursor);
    }while(this->data_->current_move(1) && ++cursor < total_recs);

    this->data_->current(current_rec);
    this->grid_->EndBatch();
}


void
wxPg::Grid_binding::prepare_grid_(){
    int total_cols, total_rows;

    if(this->col_bindings_.size())
        total_cols = (--(this->col_bindings_.end()))->first + 1;
    else total_cols = this->data_->cols();
    total_rows = this->data_->rows();

    if(this->grid_->GetTable()){
        int curr_rows = this->grid_->GetNumberRows();
        int curr_cols = this->grid_->GetNumberCols();
        if(total_cols != curr_cols){
            if(curr_cols < total_cols)
                this->grid_->AppendCols(total_cols - curr_cols);
            else this->grid_->DeleteCols(curr_cols
                                         - (curr_cols - total_cols),
                                         curr_cols - total_cols);
        }
        if(total_rows != curr_rows){
            if(curr_rows < total_rows)
                this->grid_->AppendRows(total_rows - curr_rows);
            else this->grid_->DeleteRows(curr_rows
                                         - (curr_rows - total_rows),
                                         curr_rows - total_rows);
        }
    }else this->grid_->CreateGrid(total_rows, total_cols);
    for(int i = 0; i < total_cols; ++i)
        this->grid_->SetColLabelValue(i,
                this->col_label_(this->data_->col_name(i)));
}


const wxString
wxPg::Grid_binding::col_label_(const wxString &field){
    std::map<wxString, wxString>::iterator it = this->col_labels_.find(field);
    if(it != this->col_labels_.end())
        return it->second;

    return field;
}


void
wxPg::Grid_binding::fill_row_(int row){
    for(std::map<int, wxString>::iterator it = this->col_bindings_.begin();
            it != this->col_bindings_.end(); ++it)
        this->grid_->SetCellValue(row, it->first,
                                  this->data_->get_str(it->second));
}


void
wxPg::Grid_binding::fill_row_all_(int row){
    int total_cols = this->data_->cols();
    for(int i = 0; i < total_cols; ++i)
        this->grid_->SetCellValue(row, i, this->data_->get_str(i));
}
