/* query.cpp - implementation of class wxPg::Query for executing
    SQL queries

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/db/query.h>


wxPg::Query::Query(wxPg::Database *db)
    : Data_row(db), Result_data(db),
      PORTAL_NAME(wxPg::Query::gen_portal_name()), active_(false)
{}


wxPg::Query::~Query(){
    this->clear();
}


void
wxPg::Query::clear(){
    if(!this->active_)
        return;

    this->wxPg::Result_data::clear();  //clear current result
    wxString sql = wxT("CLOSE ");
    sql += PORTAL_NAME;
    this->database()->run(sql);
    this->database()->commit();

    this->active_ = false;
}


bool
wxPg::Query::open(const wxString &sql){
    this->clear();
    wxString statement = wxT("DECLARE ");
    statement += PORTAL_NAME;
    statement += wxT(" SCROLL CURSOR FOR ");
    statement += sql;

    bool continue_trans = this->database()->in_transaction();
    if(!this->database()->enter_transaction())
        return false;
    if(!this->database()->run(statement)){
        if(continue_trans)
            this->database()->commit();  //just leave transaction
        else this->database()->rollback();  //rollback started tans.
        return false;
    }

    this->database()->attach(this, wxPg::Database::ATTACH_TRANSACTION);
    this->active_ = true;
    return true;
}


const wxString
wxPg::Query::last_err() const{
    return this->database()->last_err();
}


bool
wxPg::Query::fetch(int rows, int flag){
    if(!this->active_)
        return false;

    wxString sql = this->fetch_move_(wxT("FETCH "), rows, flag);
    if(sql == wxEmptyString)
        return false;

    this->wxPg::Result_data::clear();
    this->result_ = this->database()->run_(sql);
    if(!this->result_)
        return false;
    this->current_ = 0;

    return true;
}


bool
wxPg::Query::move(int rows, int flag){
    if(!this->active_)
        return false;

    wxString sql = this->fetch_move_(wxT("MOVE "), rows, flag);
    if(sql == wxEmptyString)
        return false;

    this->wxPg::Result_data::clear();
    this->result_ = this->database()->run_(sql);
    this->current_ = 0;
    return this->result_;
}


const wxString
wxPg::Query::fetch_move_(const wxString &prefix, int rows, int flag){
    bool use_count = false;
    wxString sql = prefix;
    switch(flag){
    case FM_FIRST:
        sql += wxT("FIRST ");
        use_count = false;
        break;
    case FM_LAST:
        sql += wxT("LAST ");
        use_count = false;
        break;
    case FM_ABSOLUTE:
        sql += wxT("ABSOLUTE ");
        use_count = true;
        break;
    case FM_RELATIVE:
        sql += wxT("RELATIVE ");
        use_count = true;
        break;
    case FM_ALL:
        sql += wxT("ALL ");
        use_count = false;
        break;
    case FM_FORWARD:
        sql += wxT("FORWARD ");
        use_count = true;
        break;
    case FM_FORWARD_ALL:
        sql += wxT("FORWARD ALL ");
        use_count = false;
        break;
    case FM_BACKWARD:
        sql += wxT("BACKWARD ");
        use_count = true;
        break;
    case FM_BACKWARD_ALL:
        sql += wxT("BACKWARD ALL ");
        use_count = false;
        break;
    default:
        return wxEmptyString;
    }

    if(use_count)
        sql << rows;
    sql += wxT(" FROM ");
    sql += PORTAL_NAME;

    return sql;
}


unsigned
wxPg::Query::portal_id_gen;


const wxString
wxPg::Query::gen_portal_name(){
    unsigned id = ++wxPg::Query::portal_id_gen;
    wxString result = wxT("wxPG_Query_portal");
    result << id;
    return result;
}
