/* query_browser.cpp - implementation of class wxPg::Query_browser.

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/db/query_browser.h>


wxPg::Query_browser::Query_browser(Database *db, unsigned buf_size)
    : Data_row(db), wxPg::Query(db), buffer_size_(buf_size), abs_pos_(0),
      total_recs_(0), abs_cursor_pos_(-1), end_reached_(false)
{}


wxPg::Query_browser::~Query_browser(){}


void
wxPg::Query_browser::buffer_size(unsigned size){
    this->buffer_size_ = size;
}


unsigned
wxPg::Query_browser::buffer_size()const{
    return this->buffer_size_;
}


/* When _abs_pos == _current, currently fetched buffer is the beginning
    of result. Othrwise we need to move to beginning and fetch buffer of
    rows.
    At the end set position variables to 0, to activate the first row.
*/
void
wxPg::Query_browser::first(){
    if(static_cast<long long>(this->abs_pos_)
            != static_cast<long long>(this->current_)){
        if(!this->Query::move(0, FM_ABSOLUTE))
            return;
        if(!this->Query::fetch(this->buffer_size_))
            return;
        this->abs_cursor_pos_ = this->rows() - 1;
    }
    this->abs_pos_ = 0;
    this->current_ = 0;
    if(this->abs_cursor_pos_ >= static_cast<long long>(this->total_recs_)){
        this->total_recs_ = this->abs_cursor_pos_ + 1;
        this->end_reached_ = true;
    }
}


/*
IF _end_reached THEN
    fetch last _buffer_size of rows
    set active row to last in buffer
ELSE
    move to last row and get number of rows in result.
    move backwards by _buffer_size
    fetch last _buffer_size of rows
    set active row to last in buffer
ENDIF
*/
void
wxPg::Query_browser::last(){
    if(this->end_reached_){
        if(this->abs_pos_ == this->total_recs_ - 1)
            return;
        if(this->abs_cursor_pos_
                >= static_cast<long long>(this->total_recs_) - 1){
            this->current_ = this->rows() - 1;
            this->abs_pos_ = this->total_recs_ - 1;
        }else{
            if(!this->Query::move(this->total_recs_ - this->buffer_size_,
                                  FM_ABSOLUTE))
                return;
            if(!this->Query::fetch(this->buffer_size_))
                return;
            this->abs_pos_ = this->total_recs_ - 1;
            this->abs_cursor_pos_ = this->abs_pos_;
            this->current_ = this->rows() - 1;
        }
    }else{
        if(!this->Query::move(0, FM_FORWARD_ALL))
            return;
        this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_), NULL, 10);
        this->abs_pos_ = this->abs_cursor_pos_;
        this->total_recs_ = this->abs_cursor_pos_ + 1;
        this->end_reached_ = true;
        if(!this->Query::move(this->buffer_size_ + 1, FM_BACKWARD))
            return;
        if(!this->Query::fetch(this->buffer_size_))
            return;
        this->current_ = this->rows() - 1;
    }
}


/*
Check for end and return if true.
If in the midle of buffer, increment position and return.
Fetch next buffer
    go to last on error or no rows.
    update position state variables.
*/
bool
wxPg::Query_browser::next(){
    if(this->end_reached_ && this->abs_pos_ == this->total_recs_ - 1)
        return false;
    if(this->current_ < this->rows() - 1){ //inside current buffer
        ++this->current_;
        ++this->abs_pos_;
    }else{
        if(!this->Query::fetch(this->buffer_size_)){
            this->last();
            return false;
        }else{
            int rows = this->rows();
            if(!rows){
                this->last();
                return false;
            }
            this->abs_cursor_pos_ += rows;
            if(this->abs_cursor_pos_ >= 0)
                ++this->abs_pos_;
            if(this->abs_cursor_pos_ >= static_cast<long long>(
                    this->total_recs_))
                this->total_recs_ = this->abs_cursor_pos_ + 1;
            if(unsigned(rows) < this->buffer_size_){
                ++this->abs_cursor_pos_;
                this->end_reached_ = true;
            }
        }
    }

    return true;
}


/*
Check for beginning of result.
If in the midle of buffer, decrement position variables.
Move backwards by all current buffer + _buffer_size.
Fetch buffer.
Update position variables.
*/
bool
wxPg::Query_browser::prev(){
    if(!this->abs_pos_)
        return false;
    if(this->current_){ //inside current buffer
        --this->current_;
        --this->abs_pos_;
    }else{
        int fetch_from = int(this->abs_cursor_pos_ - this->rows()
                             - this->buffer_size_ + 1);
        if(fetch_from < 0)
            fetch_from = 0;
        if(!this->Query::move(fetch_from, FM_ABSOLUTE)){
            this->first();
            return false;
        }
        if(!this->Query::fetch(this->buffer_size_)){
            return false;
        }

        this->abs_cursor_pos_ = fetch_from + this->rows() - 1;
        --this->abs_pos_;
        this->current_ = this->rows()
                         - (this->abs_cursor_pos_ - this->abs_pos_ + 1);
    }

    return true;
}



/*
Call parents open().
Fetch _buffer_size of rows.
Update position variables.
*/
bool
wxPg::Query_browser::open(const wxString &sql){
    //simple open from parent
    if(!this->Query::open(sql))
        return false;
    //need simple fetch operation, implemented in parent
    if(!this->Query::fetch(this->buffer_size_))
        return false;
    this->abs_pos_ = 0;
    this->abs_cursor_pos_ = this->rows() - 1;
    this->total_recs_ = this->rows();
    this->end_reached_ = false;
    return true;
}



bool
wxPg::Query_browser::fetch(int rows, int flag){
    switch(flag){
    case Query::FM_FIRST:
        return this->fetch_first_();
    case Query::FM_LAST:
        return this->fetch_last_();
    case Query::FM_ABSOLUTE:
        //we use 0-based indexing, not 1-based
        return this->fetch_absolute_(rows + 1);
    case Query::FM_RELATIVE:
        return this->fetch_relative_(rows);
    case Query::FM_FORWARD:
        return this->fetch_forward_(rows);
    case Query::FM_ALL:
    case Query::FM_FORWARD_ALL:
        return this->fetch_forward_all_();
    case Query::FM_BACKWARD:
        return this->fetch_backward_(rows);
    case Query::FM_BACKWARD_ALL:
        return this->fetch_backward_all_();
    }

    return false;
}


/*
Call parents move().
Update position variables.
*/
bool
wxPg::Query_browser::move(int rows, int flag){
    if(!this->Query::move(rows, flag))
        return false;
    unsigned long long rows_moved = strtoull(PQcmdTuples(this->result_), NULL, 0);

    switch(flag){
    case Query::FM_FIRST:
        this->abs_cursor_pos_ = 0;
        this->abs_pos_ = 0;
        break;
    case Query::FM_LAST:
        this->abs_cursor_pos_ = this->abs_cursor_pos_ + rows_moved;
        this->abs_pos_ = this->abs_cursor_pos_;
        if(!this->end_reached_){
            this->total_recs_ = this->abs_cursor_pos_ + 1;
            this->end_reached_ = true;
        }
        break;
    case Query::FM_ABSOLUTE:
        this->abs_pos_ = this->abs_cursor_pos_ = rows;
        break;
    case Query::FM_RELATIVE:
    case Query::FM_FORWARD:
        this->abs_cursor_pos_ += rows_moved;
        this->abs_pos_ = this->abs_cursor_pos_;
        break;
    case Query::FM_BACKWARD:
    case Query::FM_BACKWARD_ALL:
        this->abs_cursor_pos_ -= rows_moved;
        this->abs_pos_ = this->abs_cursor_pos_;
        break;
    }

    return true;
}


/*
IF parents current() succeeds THEN
    update _abs_pos
ENDIF
*/
bool
wxPg::Query_browser::current(int new_curr){
    int diff = new_curr - this->current_;
    if(this->Query::current(new_curr)){
        this->abs_pos_ += diff;
        return true;
    }

    return false;
}


unsigned long long
wxPg::Query_browser::abs_pos()const{
    return this->abs_pos_;
}


long long
wxPg::Query_browser::cursor_pos()const{
    return this->abs_cursor_pos_;
}


bool
wxPg::Query_browser::is_last()const{
    return this->end_reached_ && this->abs_pos_ == this->total_recs_;
}


unsigned long long
wxPg::Query_browser::record_count()const{
    return this->total_recs_;
}


/*
Call first() using _buffer_size == 1.
*/
bool
wxPg::Query_browser::fetch_first_(){
    int tmp = this->buffer_size_;
    this->buffer_size_ = 1;
    this->first();
    this->buffer_size_ = tmp;
    return this->rows();
}


/*
Call last() using _buffer_size == 1.
*/
bool
wxPg::Query_browser::fetch_last_(){
    int tmp = this->buffer_size_;
    this->buffer_size_ = 1;
    this->last();
    this->buffer_size_ = tmp;
    return this->rows();
}


/*
IF requested row beyond known limits THEN
    move to requested position, false on error.
    updateposition variables
    IF move to requested position succeeded THEN
        fetch requested row and return result of operation.
    ELSE
        requested rows not exists, return false.
    ENDIF
ELSE
    fetch requested row.
    update position.
ENDIF
*/
bool
wxPg::Query_browser::fetch_absolute_(int row){
    //we use 0-based indexing, not 1-based
    if(row > int(this->total_recs_)){
        if(!this->Query::move(row, FM_ABSOLUTE))
            return false;
        this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_), NULL, 0);
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        if(this->abs_cursor_pos_ >= static_cast<long long>(this->total_recs_))
            this->total_recs_ = this->abs_cursor_pos_ + 1;
        if(int(this->abs_cursor_pos_) == row - 1)
            return this->Query::fetch(row, FM_ABSOLUTE);
        this->end_reached_ = true;
        return false;
    }
    else{
        if(!this->Query::fetch(row, FM_ABSOLUTE))
            return false;
        this->abs_pos_ = this->abs_cursor_pos_ = row - 1;
        return true;
    }
}


bool
wxPg::Query_browser::fetch_relative_(int offset){
    return this->fetch_absolute_(this->abs_cursor_pos_ + offset + 1);
}


/*
Fetch rows.
Update position.
*/
bool
wxPg::Query_browser::fetch_forward_(int rows){
    if(!this->Query::fetch(rows, FM_FORWARD))
        return false;
    int fetched = this->rows();
    if(fetched == 0){
        this->end_reached_ = true;
        return false;
    }
    this->abs_cursor_pos_ += fetched;
    this->abs_pos_ = this->abs_cursor_pos_ - fetched + 1;
    if(this->abs_cursor_pos_ >= static_cast<long long>(this->total_recs_))
        this->total_recs_ = this->abs_cursor_pos_ + 1;
    if(fetched < rows){
        this->end_reached_ = true;
        ++this->abs_cursor_pos_;
    }
    return true;
}


/*
Fetch all foreward.
Update position.
*/
bool
wxPg::Query_browser::fetch_forward_all_(){
    if(!this->Query::fetch(0, FM_FORWARD_ALL))
        return false;

    unsigned long long fetched = strtoull(PQcmdTuples(this->result_), NULL, 0);
    if(fetched){
        this->abs_cursor_pos_ += fetched;
        this->abs_pos_ = this->abs_cursor_pos_;
        this->total_recs_ = this->abs_cursor_pos_ + 1;
    }else this->abs_cursor_pos_ = this->total_recs_;

    this->end_reached_ = true;
    return true;
}


/*
Move backward by rows + 1.
Fetch rows.
*/
bool
wxPg::Query_browser::fetch_backward_(int rows){
    if(this->abs_cursor_pos_ <= 0)
        return false;

    if(!this->Query::move(rows + 1, FM_BACKWARD))
        return false;
    int moved = strtol(PQcmdTuples(this->result_), NULL, 0);
    if(moved <= rows)
        rows = moved;
    if(rows <= 0)
        return false;
    if(!this->Query::fetch(rows))
        return false;

    --this->abs_cursor_pos_;
    if(this->abs_cursor_pos_ < 0){
        this->abs_cursor_pos_ = this->rows() - 1;
        this->abs_pos_ = 0;
        this->current_ = 0;
    }else{
        this->abs_pos_ = this->abs_cursor_pos_;
        this->current_ = this->rows() - 1;
    }

    return true;
}


/*
Move to position 0.
Fetch all befere previous current record.
*/
bool
wxPg::Query_browser::fetch_backward_all_(){
    if(!this->abs_cursor_pos_)
        return false;
    if(!this->Query::move(0, FM_ABSOLUTE))
        return false;
    if(!this->Query::fetch(int(this->abs_pos_)))
        return false;

    --this->abs_pos_;
    this->abs_cursor_pos_ = this->abs_pos_;
    this->current_ = this->rows() - 1;
    return true;
}
