/* utils.cpp - various utility functions

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/conf.h>
#include<wxPg/utils/utils.h>
#include<wxPg/utils/type.h>
#include<cstring>


const wxString
wxPg::escape_string(wxPg::Database &db, const wxString &s, wxString *error){
    PGconn *conn = db.handle();
    if(!conn)
        return wxEmptyString;

    int from_len = s.Len() * sizeof(wxChar) + 1;
    int to_len = from_len * 2 + 1;
    char from[from_len], to[to_len];

    std::strcpy(from, s.char_str());
    int err;
    PQescapeStringConn(conn, to, from, from_len, &err);

    if(err && error)
        *error = wxString::From8BitData(PQerrorMessage(conn));

    return wxString::From8BitData(to);
}


void
wxPg::cache_result_data(wxPg::Cached_result &dest, wxPg::Result_data &src){
    wxPg::Database *db = src.database();
    if(!db)
        return;
    //existence of result is checked inside
    dest.store(*db, src.result());
}


bool
wxPg::cstr_to_bool(char *str, int type){
    if(!str || str[0] == 0)
        return false;
    switch(type){
    case wxPg::TYPE_BOOL:
        return str[0] == 't';
    case wxPg::TYPE_INT:
        return atoi(str);
    case wxPg::TYPE_FLOAT:
        return atof(str);
    case wxPg::TYPE_STRING:
        return true;           //we get here only if string is non empty
    }

    return false;
}
