/* binding_ctrl.h - helper functions for binding wxWindow to
    wxPg::Row_binding

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __BINDING_CTRL_H
#define __BINDING_CTRL_H

#include "wxPg/conf.h"
#include "wxPg/binding/row_binding.h"

namespace wxPg{

/** Set value of wxCalendarCtrl.
    @param calendar - wxCalendarCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_calendar_ctrl(wxWindow *calendar, const wxString &field,
                     Data_row_provider *row);


/** Set value of wxComboBox.
    @param combo - wxComboBox to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_combo_box(wxWindow *combo, const wxString &field,
                 Data_row_provider *row);


/** Set value of wxComboCtrl.
    @param combo - wxComboCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_combo_ctrl(wxWindow *combo, const wxString &field,
                  Data_row_provider *row);


/** Set value of wxRichTextCtrl to value of field.
    @param text_ctrl - wxRichTextCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_rich_text_ctrl(wxWindow *text_ctrl, const wxString &field,
                      Data_row_provider *row);


/** Set value of wxTextCtrl to value of field.
    @param text_ctrl - wxTextCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_text_ctrl(wxWindow *text_ctrl, const wxString &field,
                 Data_row_provider *row);


/** Set value of wxSpinCtrl to value of field.
    @param spin - wxSpinCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_spin_ctrl(wxWindow *spin, const wxString &field,
                 Data_row_provider *row);


/** Set value of wxStaticText to value of field.
    @param static_text - wxStaticText to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_static_text(wxWindow *static_text, const wxString &field,
                   Data_row_provider *row);


/** Set value of wxHyperlinkCtrl to value of field.
    @param hyperlink - wxHyperlinkCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_hyperlink_ctrl(wxWindow *hyperlink, const wxString &field,
                      Data_row_provider *row);


/** Set value of wxDatePickerCtrl to value of field.
    @param date_ctrl - wxDatePickerCtrl to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_date_picker_ctrl(wxWindow *date_ctrl, const wxString &field,
                        Data_row_provider *row);

/** Set value of wxCheckBox to value of field.
    @param check_box - wxCheckBox to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_check_box(wxWindow *check_box, const wxString &field,
                 Data_row_provider *row);


/** Set value of wxGauge to value of field.
    @param gauge - wxGauge to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_gauge(wxWindow *gauge, const wxString &field,
             Data_row_provider *row);

/** Set value of wxSpinButton to value of field.
    @param spin - wxSpinButton to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_spin_button(wxWindow *spin, const wxString &field,
                   Data_row_provider *row);

/** Set value of wxItemContainer to value of field.
    @param container - wxItemContainer to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_item_container(wxWindow *container, const wxString &field,
                      Data_row_provider *row);

/** Set value of wxRadioBox to value of field.
    @param box - wxRadioBox to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_radio_box(wxWindow *box, const wxString &field,
                 Data_row_provider *row);

/** Set value of wxVListBox to value of field.
    @param list - wxVListBox to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_vlist_box(wxWindow *list, const wxString &field,
                 Data_row_provider *row);


/** Set value of wxHtmlWindow to value of field.
    @param html - wxHtmlWindow to update.
    @param field - database field name.
    @param row - source to obtain data.
*/
wxPG_DECLSPEC void
update_html_window(wxWindow *html, const wxString &field,
                   Data_row_provider *row);

} //namespace wxPg

#endif
