/* list_binding.h - binding wxItemContainer to contain the values of some
    field in wxPg::Result_data

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LIST_BINDING_H
#define __LIST_BINDING_H

#include "wxPg/conf.h"
#include "wxPg/binding/binding.h"
#include "wxPg/db/result_data_provider.h"
#include<map>
#include<wx/ctrlsub.h>

namespace wxPg{

/** Binding containers to contain values from entire column in
    wxPg::Result_data_provider.
*/
class wxPG_DECLSPEC
List_binding : public Binding{
public:
    /** Function, that should append values to container.
        Parametters are: container to be updated, field, binded to it
        and result data, where the data shoud be obtained.\n
        Result data shoud be left unchanged
        (restore current row to original position at the end).
    */
    typedef void (*append_ctrl_f)(wxItemContainer*, const wxString&,
                                  Result_data_provider*);

    /** Creates new object.
        Call data() to complete initialization.
    */
    List_binding();

    /** Creates new object.
        @param data will be used to update containers.
    */
    explicit
    List_binding(Result_data_provider *data);

    /** Destroyes the object. */
    virtual
    ~List_binding(){}

    /** Set result data, used to update containers.
        @param data - new result data to be used.
    */
    void
    data(Result_data_provider *data);

    /** Return currently used data to update containers.
        @return currently used data to update containers.
    */
    Result_data_provider*
    data();

    /** Bind container to contain values from field.
        Usualy you don't need to call this method directly -
        use binding macros, defined in binding_macros.h.
        @param c - container to be binded.
        @param field - database field to be binded to container.
        @param f is function, responsible for updating container.
    */
    void
    bind(wxItemContainer *c, wxString field, append_ctrl_f f);

    /** Remove container from bindings.
        @param c is no longer updated.
    */
    virtual void
    unbind(wxItemContainer *c);

    /** Updates UI.
        For each binded container a specified function is called.
    */
    virtual void
    update_UI();
protected:
    Result_data_provider*
    data_;

    typedef std::pair<wxString, append_ctrl_f>
    ctrl_binding;

    std::map<wxItemContainer*, ctrl_binding>
    ctrl_bindings_;

    /* Inherited. */
    virtual void
    unbind(wxWindow *c);
private:
    List_binding(const List_binding&){}
    List_binding &operator=(const List_binding&){return *this;}
};

}


#endif
