/* ui_manager.h - class for updating user interface.

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __UI_MANAGER_H
#define __UI_MANAGER_H

#include "wxPg/conf.h"
#include "wxPg/binding/ui_updater.h"
#include<set>

namespace wxPg{

/** A container class for updating UI by multiple UI_updaters at once. */
class wxPG_DECLSPEC
UI_manager : public UI_updater{
public:
    /** Constructor - creates new object. */
    UI_manager();

    /** Destructor. */
    virtual
    ~UI_manager();

    /** Update user interface.
        Calls update_UI() for all contained UI_updaters.
    */
    virtual void
    update_UI();

    /** Add updater to this container, if it't not yet contained.
        It's safe to call this method multiple times with same argument -
        same UI_updater only once will be added.
        @param updater - an UI_updater to add.
    */
    void
    add(UI_updater *updater);

    /** Remove updater from this container.
        Nothing happens, if updater is not contained by this container.
        @param updater - an UI_updater to remove.
    */
    void
    remove(UI_updater *updater);

    /** Remove all contained UI_updaters. */
    void
    clear();
private:
    //sub-updaters to manage
    std::set<UI_updater*>
    updaters_;

    UI_manager(const UI_manager&){}
    UI_manager operator=(const UI_manager&){return *this;}
};

}

#endif
