/* data_row.h - class wxPg::Data_row for accessing data from single
    row of SQL query result

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DATA_ROW_H
#define __DATA_ROW_H

#include "wxPg/conf.h"
#include "wxPg/db/db_client.h"
#include "wxPg/db/database.h"
#include "wxPg/db/data_row_provider.h"
#include<wx/string.h>
#include<wx/datetime.h>


namespace wxPg{

/** Class, representing a single row of SQL query result. */
class wxPG_DECLSPEC
Data_row : virtual public DB_client, virtual public Data_row_provider{
public:
    /** Creates new object.
        You shouldn't need to create Data_row object yourself.
        @param db - database connection.
    */
    explicit
    Data_row(Database *db);

    /** Destroys object. */
    virtual
    ~Data_row();

    /** Get database for this object.
        @return database connection for this object.
    */
    Database*
    database()const{return this->db_;}

    /** Change database connection for this object. */
    void
    database(Database *db){this->db_ = db;}

    //inherited
    virtual void
    detach();

    /** Set PGresult for the object. Used internally. */
    void
    result(PGresult *res){this->result_ = res;}

    /** Returns PGresult of the object. */
    PGresult*
    result(){return this->result_;}

    /** Clears result, frees resources. */
    virtual void
    clear();

    //inherited
    virtual int
    cols();

    //inherited
    virtual const wxString
    col_name(int idx);

    //inherited
    virtual int
    col_idx(const wxString &field);

    //inherited
    virtual int
    type_simple(int idx);

    //inherited
    virtual int
    type_simple(const wxString &field);

    //inherited
    virtual bool
    null(int idx);

    //inherited
    virtual bool
    null(const wxString &field);

    //inherited
    virtual int
    get_int(int idx);

    //inherited
    virtual int
    get_int(const wxString &field);

    //inherited
    virtual double
    get_float(int idx);

    //inherited
    virtual double
    get_float(const wxString &field);

    //inherited
    virtual const wxString
    get_str(int idx);

    //inherited
    virtual const wxString
    get_str(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date(int idx);

    //inherited
    virtual const wxDateTime
    get_date(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_time(int idx);

    //inherited
    virtual const wxDateTime
    get_time(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date_time(int idx);

    //inherited
    virtual const wxDateTime
    get_date_time(const wxString &field);

    //inherited
    virtual bool
    get_bool(int idx);

    //inherited
    virtual bool
    get_bool(const wxString &field);
protected:
    //current row in result
    int
    current_;

    //curent result
    PGresult*
    result_;
private:
    //check if column index is in range
    bool
    check_col_idx_(int);

    //check if row in in range
    bool
    check_row_idx_();

    //no copy-constructor and assignement operator
    Data_row(const Data_row&){}
    Data_row &operator=(const Data_row&){return *this;}

    //database connection
    Database*
    db_;
};

}

#endif
