/* result_data_provider.h - class wxPg::Result_data_provider, interface
    for accessing data from result of SQL query.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __RESULT_DATA_PROVIDER_H
#define __RESULT_DATA_PROVIDER_H

#include "wxPg/conf.h"
#include "wxPg/db/data_row_provider.h"

namespace wxPg{

/** Interface, representing result of SQL query. */
class wxPG_DECLSPEC
Result_data_provider : virtual public Data_row_provider{
public:
    /** Destructor. */
    virtual
    ~Result_data_provider(){}

    /** Get number of rows.
        @return number of rows in result.
    */
    virtual int
    rows() = 0;

    /** Set current row.
        @param idx - index of row to become current.
        @return true on success, false otherwise.
    */
    virtual bool
    current(int idx) = 0;

    /** Get index of current row.
        @return index of current row.
    */
    virtual int
    current()const = 0;

    /** Change current row.
        @param rows - Number of rows to move forward
        (use negative numbers to move backwards).
        @return true on success, false otherwise.
    */
    bool
    current_move(int rows){return this->current(this->current() + rows);}
};

}

#endif

