/* type.h - functions for getting type information for database data types

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __TYPE_H
#define __TYPE_H

#include "wxPg/conf.h"

/** wxPostgres namespace. */
namespace wxPg{

/** \file
    SQL datatype representation in wxPostgres.
*/


/** Unknown data type. */
const int TYPE_UNKNOWN = 0;
/** Represents int2 and int4 types. */
const int TYPE_INT = 1;
/** Represents float4 and float8 types. */
const int TYPE_FLOAT = 2;
/** Represents char, name and varchar types. */
const int TYPE_STRING = 3;
/** Represents date type. */
const int TYPE_DATE = 4;
/** Represents time type. */
const int TYPE_TIME = 5;
/** Represents timestamp type. */
const int TYPE_TIMESTAMP = 6;
/** Represents boolean type. */
const int TYPE_BOOL = 7;


/** Converts OID to type descriptor.
    @return one of type constants, defined in type.h.
*/
wxPG_DECLSPEC int
oid_to_type_simple(int oid);

}

#endif
