/* utils.h - various utility functions

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __UTILS_H
#define __UTILS_H

#include "wxPg/conf.h"
#include "wxPg/db/database.h"
#include "wxPg/db/result_data.h"


namespace wxPg{

/** \file
    wxPostgres utilities.
*/

/** Escapes string.
    This function is a wrapper around PQescapeStringConn(), see PostgreSQL
    documentation for more info.
    @param db - a database connection.
    @param s - a string to be escaped.
    @param error - a pointer to string to write error to, can be NULL.
    @return escaped string, that can be safely used in SQL statements.
*/
wxPG_DECLSPEC const wxString
escape_string(Database &db, const wxString &s, wxString *error = NULL);


/** Cache query result locally.
    @param dest - destination, where to store data.
    @param src - source to be stored.
*/
wxPG_DECLSPEC void
cache_result_data(Cached_result &dest, Result_data &src);


/** Convert C-string to bool.
    Used internally by library.
    See wxPg::Data_row_provider::get_bool() for details.
    @param str - C-string to convert to bool.
    @param type - one of type constants defined in type.h.
    @return str, converted to bool.
*/
wxPG_DECLSPEC bool
cstr_to_bool(char *str, int type);

}

#endif
